#!/usr/bin/perl
# Purpose: Take logs from security devices, gather stats and insert that
#          data into a mysql database.
#
# Written by: John Weidley  -- packetshack.org
# Version: 1.2
# Usage: cat <logfile> | egrep -vif <filterfile> | sensorTrends.pl
#######################################################################
# CHANGE LOG
# 18Aug03
# 	Included support for Cisco ACL IP protocol parsing.
#
# 28Jul03
# 	Split up UDP/TCP/ICMP protocols into seperate db tables for more
# 	granular trending.
#
# 30May03
# 	The original versions of the script has some serious
# 	limitations that had to be fixed with egrep pre-filtering. This 
# 	version	makes each pattern match a module.
#
# 29May03
# 	Added a function in insertDB that will NOT insert records into
# 	the database if the port total is 1. This cuts down on the
# 	number of false positives caused by misconfigured rules.
#
#######################################################################
# Configuration Section
#######################################################################
$dbhost = "localhost";
$dbuser = "root";
$dbpass = "password";
$dbname = "trends";
chomp ($year = `/bin/date +%Y`);

# End Configuration Section

#######################################################################
# Variables / Modules
#######################################################################
use DBI();
use Getopt::Long;

%TableName=('tcp'  => 'trendsTCP',
	    'udp'  => 'trendsUDP',
	    'icmp' => 'trendsICMP',
	    'ipproto'   => 'trendsIP');

%monthmap=('Jan' => '01',
           'Feb' => '02',
           'Mar' => '03',
           'Apr' => '04',
           'May' => '05',
           'Jun' => '06',
           'Jul' => '07',
           'Aug' => '08',
           'Sep' => '09',
           'Oct' => '10',
           'Nov' => '11',
           'Dec' => '12');


$usage = "Usage: cat logfile \| sensorTrends.pl\n" . 
         "       --debug - Prints the contents of the hash in a tree format\n" . 
         "       --debug_sql - Prints the SQL insert statements\n" .
         "       --help - Prints this\n";

my $dbh = DBI->connect("DBI:mysql:database=$dbname;host=$dbhost", $dbuser, $dbpass, {'RaiseError' => 1});

#######################################################################
# Options
#######################################################################
GetOptions ( "debug"            =>      \$debug,
             "debug_sql"        =>      \$debug_sql,
	     "help"		=>	\$help);

if ($help) {
	print $usage;
	exit;
}

#######################################################################
# Main
#######################################################################
while (<>) {
	@parsedLine = split;
	
	if (($parsedLine[9] =~ /SEC-6-IPACCESSLOGP/) && 
	   ($parsedLine[13] =~ /tcp|udp/i)) {
		aclParser();			### Cisco ACL format (TCP/UDP)
	}
	elsif (($parsedLine[9] =~ /SEC-6-IPACCESSLOGDP/) && 
	      ($parsedLine[13] =~ /icmp/i)) {
		aclICMPParser();		### Cisco ACL ICMP format
	}
	elsif (($parsedLine[9] =~ /SEC-6-IPACCESSLOGNP/) && 
	      ($parsedLine[13] =~ /\d+/)) {
		aclIPParser();			### Cisco ACL IP format
	}
	elsif (($parsedLine[4] =~ /PIX-4-106023/) && 
	      ($parsedLine[6] =~ /tcp|udp/i)) {
		pixParser();			### Cisco PIX format (TCP/UDP)
	}
	elsif (($parsedLine[4] =~ /PIX-4-106023/) && 
	      ($parsedLine[6] =~ /icmp/i)) {
		pixICMPParser();		### Cisco PIX ICMP format
	}
	elsif (($parsedLine[5] =~ /NetScreen/i) &&
	      ($parsedLine[13] =~ /proto\=6|proto\=17/)) {
		netscreenParser();		### NetScreen format (TCP/UDP)
	}
	elsif (($parsedLine[5] =~ /NetScreen/i) &&
	      ($parsedLine[12] =~ /service\=icmp/i)) {
		netscreenICMPParser();		### NetScreen ICMP format
	}
	elsif ($parsedLine[4] =~ /-\>/) {
		snortParser();			### Snort Portscan format
	}
}

insertDB();

#######################################################################
#######################################################################
### Modules
#######################################################################
#######################################################################

######################################
### Cisco Router ACL Parser
######################################
sub aclParser () {
	$month = $parsedLine[0];
	$day = $parsedLine[1];
	$date = $year . "-" . $monthmap{$month} . "-" . $day;
	$proto = $parsedLine[13];
	$src_pair = $parsedLine[14];
	$dst_pair = $parsedLine[16];
	($sip,$sport) = split(/\(/, $src_pair);
	($dip,$dport) = split(/\(/, $dst_pair);
	$sport =~ s/\),//;
	$dport =~ s/\),//;
	countIt();
}

######################################
### Cisco Router ACL ICMP Parser
######################################
sub aclICMPParser () {
	$month = $parsedLine[0];
	$day = $parsedLine[1];
	$date = $year . "-" . $monthmap{$month} . "-" . $day;
	$proto = $parsedLine[13];
	$sip = $parsedLine[14];
	$dip = $parsedLine[16];
	$dport = $parsedLine[17];
	$dport =~ s/\(//;
	$dport =~ s/\),//;
	countIt();
}

######################################
### Cisco Router ACL IP Parser
######################################
sub aclIPParser () {
	$month = $parsedLine[0];
	$day = $parsedLine[1];
	$date = $year . "-" . $monthmap{$month} . "-" . $day;
	$dport = $parsedLine[13];
	$proto = "ipproto";
	$sip = $parsedLine[14];
	$dip = $parsedLine[16];
	countIt();
}

######################################
### Cisco PIX Parser
######################################
sub pixParser () {
	$month = $parsedLine[0];
	$day = $parsedLine[1];
	$date = $year . "-" . $monthmap{$month} . "-" . $day;
	$proto = $parsedLine[6];
	$src_trio = $parsedLine[8];
	$dst_trio = $parsedLine[10];
	$src_pair = (split(/:/, $src_trio))[1];
	$dst_pair = (split(/:/, $dst_trio))[1];
	($sip,$sport) = split(/\//, $src_pair);
	($dip,$dport) = split(/\//, $dst_pair);
	countIt();
}

######################################
### Cisco PIX ICMP Parser
######################################
sub pixICMPParser () {
	$month = $parsedLine[0];
	$day = $parsedLine[1];
	$date = $year . "-" . $monthmap{$month} . "-" . $day;
	$proto = $parsedLine[6];
	$src_trio = $parsedLine[8];
	$dst_trio = $parsedLine[10];
	$sip = (split(/:/, $src_trio))[1];
	$dip = (split(/:/, $dst_trio))[1];
	$type = $parsedLine[12];
	$type =~ s/,//;
	$code = $parsedLine[14];
	$code =~ s/\)//;
	$dport = $type . "/" . $code;
	countIt();
}

######################################
### NetScreen Parser
######################################
sub netscreenParser () {
	$month = $parsedLine[0];
	$day = $parsedLine[1];
	$date = $year . "-" . $monthmap{$month} . "-" . $day;
	if ($parsedLine[13] eq "proto=6") {
		$proto = "tcp";
	} elsif ($parsedLine[13] eq "proto=17") {
		$proto = "udp";
	} else {
		next;
	}
	$src_pair = $parsedLine[21];
	$dst_pair = $parsedLine[22];
	$dport_pair = $parsedLine[24];
	($stitle,$sip) = split(/=/, $src_pair);
	($dtitle,$dip) = split(/=/, $dst_pair);
	($dptitle,$dport) = split(/=/, $dport_pair);
	countIt();
}

######################################
### NetScreen ICMP Parser
######################################
sub netscreenICMPParser () {
	$month = $parsedLine[0];
	$day = $parsedLine[1];
	$date = $year . "-" . $monthmap{$month} . "-" . $day;
	($svctitle,$proto) = split(/=/, $parsedLine[12]);
	$src_pair = $parsedLine[21];
	$dst_pair = $parsedLine[22];
	$dport_pair = $parsedLine[24];
	($stitle,$sip) = split(/=/, $src_pair);
	($dtitle,$dip) = split(/=/, $dst_pair);
	($dptitle,$dport) = split(/=/, $dport_pair);
	$dport = $dport . "/0";
	countIt();
}

######################################
### Snort Portscan Parser
######################################
sub snortParser () {
	$month = $parsedLine[0];
	$day = $parsedLine[1];
	$date = $year . "-" . $monthmap{$month} . "-" . $day;
	$src_pair = $parsedLine[3];
	$dst_pair = $parsedLine[5];
	($sip,$sport) = split(/:/, $src_pair);
	($dip,$dport) = split(/:/, $dst_pair);
	countIt();
}

######################################
### Update the Counter
######################################
sub countIt () {
	$mhash{$proto}{$dport}{'src'}{$sip}++;
	$mhash{$proto}{$dport}{'dst'}{$dip}++;
	$mhash{$proto}{$dport}{'total'}++;
}

######################################
### Insert into the Database
######################################
sub insertDB () {
	foreach $proto ( keys %mhash ) {
		if ($debug) { print "proto is " . $proto . "\n"; }

		if ($proto eq "icmp") {
			foreach $typeCode ( keys %{ $mhash{$proto} } ) {
				$totalsrcs = (keys %{ $mhash{$proto}{$typeCode}{'src'} });
				$totaldsts = (keys %{ $mhash{$proto}{$typeCode}{'dst'} });
				if ($debug) { print "typeCode " . $typeCode . "\n"; }
	                        if ($debug) { print "\tTotalSrcs: " . $totalsrcs . "\n"; }
                        	if ($debug) { print "\tTotalDsts: " . $totaldsts . "\n"; }
                        	if ($debug) { print "\tTotalRcds: " . $mhash{$proto}{$typeCode}{'total'} . "\n\n"; }

				# Do the SQL insert
				$rcdInsqry = sprintf("insert into %s (date,typeCode,totalRecords,totalSrcs,totalDsts)
							values ('%s','%s', '%s', '%s', '%s')",
							$TableName{$proto}, $date, $typeCode, $mhash{$proto}{$typeCode}{'total'}, $totalsrcs, $totaldsts);
				if ( $debug_sql ) { print "$rcdInsqry\n\n"; }
				if (!$debug) {
					$recordIns = $dbh->prepare($rcdInsqry) or die "Unable to query\n";
					$recordIns->execute() or die "Unable to execute query\n";
					$recordIns->finish();
				}
			}
		} else { 
			foreach $port ( keys %{ $mhash{$proto} } ) {
				$totalsrcs = (keys %{ $mhash{$proto}{$port}{'src'} });
				$totaldsts = (keys %{ $mhash{$proto}{$port}{'dst'} });
				if ($debug) { print "Port: " . $port . "\n"; }
				if ($debug) { print "\tTotalSrcs: " . $totalsrcs . "\n"; }
				if ($debug) { print "\tTotalDsts: " . $totaldsts . "\n"; }
				if ($debug) { print "\tTotalRcds: " . $mhash{$proto}{$port}{'total'} . "\n\n"; }

				# Do the SQL insert
				if ($proto eq "ipproto") {
					$field = "proto";
				} else {
					$field = "port";
				}

				$rcdInsqry = sprintf("insert into %s (date,$field,totalRecords,totalSrcs,totalDsts) 
						values ('%s','%s', '%s', '%s', '%s')", 
						$TableName{$proto}, $date, $port, $mhash{$proto}{$port}{'total'}, $totalsrcs, $totaldsts);
				if ( $debug_sql ) { print "$rcdInsqry\n\n"; }
				if (!$debug) {
					$recordIns = $dbh->prepare($rcdInsqry) or die "Unable to query\n";
					$recordIns->execute() or die "Unable to execute query\n";
					$recordIns->finish();
				}
			}
		}
	}
}


## End of file ##
